@@ -1,5 +1,5 @@
 package URI::Fast;
-$URI::Fast::VERSION = '0.18';
+$URI::Fast::VERSION = '0.19';
 # ABSTRACT: A fast(er) URI parser
 
 use utf8;
@@ -137,7 +137,7 @@ URI::Fast - A fast(er) URI parser
 
 =head1 VERSION
 
-version 0.18
+version 0.19
 
 =head1 SYNOPSIS
 
@@ -534,6 +534,10 @@ size_t uri_decode(const char *in, size_t len, char *out) {
 
   out[j] = '\0';
 
+  if (!is_utf8_string(out, j)) {
+    croak("uri_decode: invalid multibyte sequence");
+  }
+
   return j;
 }
 
@@ -579,7 +583,7 @@ SV* decode(SV* in) {
     SvUTF8_on(in);
 
     if (!sv_utf8_downgrade(in, TRUE)) {
-      croak("decode: wide character in octet string");
+      croak("decode: wide character in input octet string");
     }
 
     src = SvPV_nomg_const(in, ilen);
@@ -913,9 +917,7 @@ SV* get_param(SV* uri, SV* sv_key) {
 
       char val[brk + 1];
       vlen = uri_decode(&src[idx], brk, val);
-
       idx += brk + 1;
-
       value = newSVpv(val, vlen);
       SvUTF8_on(value);
       av_push(out, value);
